@@ -8,23 +8,10 @@ use Regexp::RegGrp;
 
 # =========================================================================== #
 
-our $VERSION = '1.005_001';
-
-our @BOOLEAN_ACCESSORS = (
-    'no_compress_comment',
-    'remove_copyright'
-);
-
-our @COPYRIGHT_ACCESSORS = (
-    'copyright',
-    'copyright_comment'
-);
-
-our @COMPRESS_OPTS      = ( 'clean', 'obfuscate', 'shrink', 'best' );
-our $DEFAULT_COMPRESS   = 'clean';
+our $VERSION = '1.001';
 
 our $PACKER_COMMENT     = '\/\*\s*JavaScript::Packer\s*(\w+)\s*\*\/';
-our $COPYRIGHT_COMMENT  = '\/\*((?>[^\*]|\*[^\/])*copyright(?>[^\*]|\*[^\/])*)\*\/';
+our $COPYRIGHT_COMMENT  = '(\/\*(?>[^\*]|\*[^\/])*copyright(?>[^\*]|\*[^\/])*\*\/)';
 
 our $SHRINK_VARS = {
     BLOCK           => qr/(((catch|do|if|while|with|function)\b[^~{};]*(\(\s*[^{};]*\s*\))\s*)?(\{[^{}]*\}))/,  # function ( arg ) { ... }
@@ -80,10 +67,12 @@ our $COMMENTS = [
         replacement => ''
     },
     {
-        regexp      => $DICTIONARY->{COMMENT1} . '\s*(' . $DICTIONARY->{REGEXP} . ')?'
+        regexp      => $DICTIONARY->{COMMENT1} . '\s*(' . $DICTIONARY->{REGEXP} . ')?',
+        replacement => sub { return sprintf( "\n%s", $_[0]->{submatches}->[2] ); }
     },
     {
-        regexp      => '(' . $DICTIONARY->{COMMENT2} . ')\s*(' . $DICTIONARY->{REGEXP} . ')?'
+        regexp      => '(' . $DICTIONARY->{COMMENT2} . ')\s*(' . $DICTIONARY->{REGEXP} . ')?',
+        replacement => sub { return sprintf( " %s", $_[0]->{submatches}->[1] ); }
     }
 ];
 
@@ -143,7 +132,7 @@ our $WHITESPACE = [
     }
 ];
 
-our $TRIM = [
+ our $TRIM = [
     {
         regexp      => '(\d)(?:\|\d)+\|(\d)',
         replacement => sub { return sprintf( "%d-%d", $_[0]->{submatches}->[0] || 0, $_[0]->{submatches}->[1] || 0 ); }
@@ -162,129 +151,129 @@ our $TRIM = [
     }
 ];
 
-our @REGGRPS = ( 'comments', 'clean', 'whitespace', 'concat', 'trim', 'data_store' );
-
-# --------------------------------------------------------------------------- #
-
-{
-    no strict 'refs';
-
-    foreach my $field ( @BOOLEAN_ACCESSORS ) {
-        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
-
-        *{ __PACKAGE__ . '::' . $field} = sub {
-            my ( $self, $value ) = @_;
-
-            $self->{'_' . $field} = $value ? 1 : undef if ( defined( $value ) );
-
-            return $self->{'_' . $field};
-        };
-    }
-
-    foreach my $field ( @COPYRIGHT_ACCESSORS ) {
-        $field = '_' . $field if ( $field eq 'copyright_comment' );
-        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
-
-        *{ __PACKAGE__ . '::' . $field} = sub {
-            my ( $self, $value ) = @_;
-
-            if ( defined( $value ) and not ref( $value ) ) {
-                $value =~ s/^\s*|\s*$//gs;
-                $self->{'_' . $field} = $value;
-            }
-
-            my $ret = '';
-
-            if ( $self->{'_' . $field} ) {
-                $ret = '/* ' . $self->{'_' . $field} . ' */' . "\n";
-            }
-
-            return $ret;
-        };
-    }
-
-    foreach my $reggrp ( @REGGRPS ) {
-        next if defined *{ __PACKAGE__ . '::reggrp_' . $reggrp }{CODE};
-
-        *{ __PACKAGE__ . '::reggrp_' . $reggrp } = sub {
-            my ( $self ) = shift;
-
-            return $self->{ '_reggrp_' . $reggrp };
-        };
-    }
-}
-
-sub compress {
-    my ( $self, $value ) = @_;
-
-    if ( defined( $value ) ) {
-        if ( grep( $value eq $_, @COMPRESS_OPTS ) ) {
-            $self->{_compress} = $value;
-        }
-        elsif ( ! $value ) {
-            $self->{_compress} = undef;
-        }
-    }
-
-    $self->{_compress} ||= $DEFAULT_COMPRESS;
-
-    return $self->{_compress};
-}
-
 sub init {
     my $class   = shift;
     my $self    = {};
 
-    bless( $self, $class );
+    map {
+        my $what = $_;
 
-    @{$self->{clean}->{reggrp_data}}        = ( @$DATA, @$CLEAN );
-    @{$self->{whitespace}->{reggrp_data}}   = ( @$DATA[ 0, 1, 3 ], @$WHITESPACE );
-    $self->{trim}->{reggrp_data}            = $TRIM;
+        map {
+            push(
+                @{$self->{$what}->{reggrp_data}},
+                {
+                    regexp      => $_->{regexp},
+                    replacement => $_->{replacement}
+                }
+            );
+        } @$DATA;
 
-    @{$self->{data_store}->{reggrp_data}}   = map {
-        {
-            regexp  => $_->{regexp},
-            store   => sub { return sprintf( "%s", $_[0]->{match} ); }
+        if ( $what eq 'comments' ) {
+            map {
+                push(
+                    @{$self->{comments}->{reggrp_data}},
+                    {
+                        regexp      => $_->{regexp},
+                        replacement => $_->{replacement}
+                    }
+                );
+            } @$COMMENTS;
         }
+        elsif ( $what eq 'clean' ) {
+            map {
+                push(
+                    @{$self->{clean}->{reggrp_data}},
+                    {
+                        regexp      => $_->{regexp},
+                        replacement => $_->{replacement}
+                    }
+                );
+            } @$CLEAN;
+        }
+        elsif ( $what eq 'whitespace' ) {
+            map {
+                push(
+                    @{$self->{whitespace}->{reggrp_data}},
+                    {
+                        regexp      => $_->{regexp},
+                        replacement => $_->{replacement}
+                    }
+                );
+            } @$WHITESPACE;
+        }
+    } ( 'comments', 'clean', 'whitespace', 'concat' );
+
+    splice( @{$self->{comments}->{reggrp_data}}, 2, 1 );
+    splice( @{$self->{whitespace}->{reggrp_data}}, 2, 1 );
+    splice( @{$self->{concat}->{reggrp_data}}, 2, 1 );
+
+    map {
+        push(
+            @{$self->{data_store}->{reggrp_data}},
+            {
+                regexp  => $_->{regexp},
+                store   => sub { return sprintf( "%s", $_[0]->{match} ); }
+            }
+        );
     } @$DATA;
 
-    $self->{data_store}->{reggrp_data}->[-1]->{replacement} = sub {
-        return sprintf( "%s\x01%d\x01", $_[0]->{submatches}->[0], $_[0]->{store_index} );
-    };
-
-    $self->{data_store}->{reggrp_data}->[-1]->{store} = sub {
-        return $_[0]->{submatches}->[1];
-    };
-
-    @{$self->{concat}->{reggrp_data}}       = map {
-        my $data = $_;
-        {
-            regexp      => '(' . $data->{regexp} . ')((?:\+' . $data->{regexp} . ')+)',
-            replacement => sub {
-                my $submatches = $_[0]->{submatches};
-                my $ret = $submatches->[0];
-
-                my $next_str = '^\+(' . $data->{regexp} . ')';
-
-                while ( my ( $next ) = $submatches->[1] =~ /$next_str/ ) {
-                    chop( $ret );
-                    $ret .= substr( $next, 1 );
-                    $submatches->[1] =~ s/$next_str//;
-                }
-
-                return $ret;
+    for ( my $i = 1; $i >= 0; $i-- ) {
+        unshift(
+            @{$self->{concat}->{reggrp_data}},
+            {
+                regexp => $DATA->[$i]->{regexp}
             }
-        };
-    } @$DATA[ 0, 1 ];
+        );
+        unshift(
+            @{$self->{concat}->{reggrp_data}},
+            {
+                regexp      => '(' . $DATA->[$i]->{regexp} . ')((?:\+' . $DATA->[$i]->{regexp} . ')+)',
+                replacement => sub {
+                    my $submatches = $_[0]->{submatches};
+                    my $ret = $submatches->[0];
 
-    @{$self->{comments}->{reggrp_data}}     = ( @$DATA[ 0, 1, 3 ], @$COMMENTS );
+                    my $next_str = '^\+(' . $DATA->[0]->{regexp} . '|' . $DATA->[1]->{regexp} . ')';
+
+                    while ( my ( $next ) = $submatches->[1] =~ /$next_str/ ) {
+                        chop( $ret );
+                        $ret .= substr( $next, 1 );
+                        $submatches->[1] =~ s/$next_str//;
+                    }
+
+                    return $ret;
+                }
+            }
+        );
+    }
+
+    push(
+        @{$self->{concat}->{reggrp_data}},
+        {
+            regexp      => $DATA->[3]->{regexp},
+            replacement => $DATA->[3]->{replacement}
+        }
+    );
+
+    map {
+        push(
+            @{$self->{trim}->{reggrp_data}},
+            {
+                regexp      => $_->{regexp},
+                replacement => $_->{replacement}
+            }
+        );
+    } @$TRIM;
 
     $self->{comments}->{reggrp_data}->[-2]->{replacement} = sub {
         my $submatches = $_[0]->{submatches};
         if ( $submatches->[0] eq '@' ) {
-            $self->reggrp_comments()->exec( \$submatches->[1] );
-            $self->reggrp_clean()->exec( \$submatches->[1] );
-            $self->reggrp_whitespace()->exec( \$submatches->[1] );
+            # I don't like this, but
+            # $self->{comments}->{reggrp}->exec( \$_[1]->[1] ); ...
+            # will not work. It isn't initialized jet.
+            # If someone has a better idea, please let me know
+            $self->_process_wrapper( 'comments', \$submatches->[1] );
+            $self->_process_wrapper( 'clean', \$submatches->[1] );
+            $self->_process_wrapper( 'whitespace', \$submatches->[1] );
 
             return sprintf( "//%s%s\n%s", @{$submatches} );
         }
@@ -295,22 +284,34 @@ sub init {
         my $submatches = $_[0]->{submatches};
         if ( $submatches->[0] =~ /^\/\*\@(.*)\@\*\/$/sm ) {
             my $cmnt = $1;
-
-            $self->reggrp_comments()->exec( \$cmnt );
-            $self->reggrp_clean()->exec( \$cmnt );
-            $self->reggrp_whitespace()->exec( \$cmnt );
+            # Same as above
+            $self->_process_wrapper( 'comments', \$cmnt );
+            $self->_process_wrapper( 'clean', \$cmnt );
+            $self->_process_wrapper( 'whitespace', \$cmnt );
 
             return sprintf( '/*@%s@*/ %s', $cmnt, $submatches->[1] );
         }
         return sprintf( " %s", $submatches->[1] );
     };
 
-    foreach ( @REGGRPS ) {
-        $self->{ '_reggrp_' . $_ } = Regexp::RegGrp->new( { reggrp => $self->{$_}->{reggrp_data} } );
-    }
+    $self->{data_store}->{reggrp_data}->[-1]->{replacement} = sub {
+        return sprintf( "%s\x01%d\x01", $_[0]->{submatches}->[0], $_[0]->{store_index} );
+    };
+
+    $self->{data_store}->{reggrp_data}->[-1]->{store} = sub {
+        return $_[0]->{submatches}->[1];
+    };
+
+    map {
+        $self->{$_}->{reggrp} = Regexp::RegGrp->new( { reggrp => $self->{$_}->{reggrp_data} } );
+    } ( 'comments', 'clean', 'whitespace', 'concat', 'trim' );
+
+    $self->{data_store}->{reggrp} = Regexp::RegGrp->new( { reggrp => $self->{data_store}->{reggrp_data} } );
 
     $self->{block_data} = [];
 
+    bless( $self, $class );
+
     return $self;
 }
 
@@ -349,44 +350,57 @@ sub minify {
         $javascript = ref( $input ) ? $input : \$input;
     }
 
-    if ( ref( $opts ) eq 'HASH' ) {
-        foreach my $field ( @BOOLEAN_ACCESSORS ) {
-            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
+    if ( ref( $opts ) ne 'HASH' ) {
+        carp( 'Second argument must be a hashref of options! Using defaults!' ) if ( $opts );
+        $opts = { compress => 'clean', copyright => '', no_compress_comment => 0, remove_copyright => 0 };
+    }
+    else {
+        $opts->{compress} ||= 'clean';
+        unless (
+            grep(
+                $_ eq $opts->{compress},
+                ( 'clean', 'minify', 'shrink', 'base62', 'obfuscate', 'best' )
+            )
+        ) {
+            $opts->{compress} = 'clean';
         }
 
-        foreach my $field ( 'compress', 'copyright' ) {
-            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
+        if ( $opts->{compress} eq 'minify' ) {
+            $opts->{compress} = 'clean';
         }
+        elsif ( $opts->{compress} eq 'base62' ) {
+            $opts->{compress} = 'obfuscate';
+        }
+
+        $opts->{remove_copyright}       = $opts->{remove_copyright} ? 1 : 0;
+        $opts->{no_compress_comment}    = $opts->{no_compress_comment} ? 1 : 0;
+        $opts->{copyright}              = '' if ( ref( $opts->{copyright} ) );
+        $opts->{copyright}              = ( $opts->{copyright} and $opts->{compress} eq 'clean' ) ? ( '/* ' . $opts->{copyright} . ' */' ) : '';
     }
 
-    my $copyright_comment = '';
-
-    if ( ${$javascript} =~ /$COPYRIGHT_COMMENT/ism ) {
-        $copyright_comment = $1;
+    if ( not $opts->{remove_copyright} and not $opts->{copyright} and ${$javascript} =~ /$COPYRIGHT_COMMENT/ism ) {
+        $opts->{copyright} = $1;
     }
-    # Resets copyright_comment() if there is no copyright comment
-    $self->_copyright_comment( $copyright_comment );
 
-    if ( not $self->no_compress_comment() and ${$javascript} =~ /$PACKER_COMMENT/ ) {
+    if ( not $opts->{no_compress_comment} and ${$javascript} =~ /$PACKER_COMMENT/ ) {
         my $compress = $1;
         if ( $compress eq '_no_compress_' ) {
-            return ${$javascript} if ( $cont eq 'scalar' );
-            return;
+            return ( $cont eq 'scalar' ) ? ${$javascript} : undef;
         }
 
-        $self->compress( $compress );
+        $opts->{compress} = grep( $compress, ( 'clean', 'shrink', 'obfuscate', 'best' ) ) ? $compress : $opts->{compress};
     }
 
     ${$javascript} =~ s/\r//gsm;
     ${$javascript} .= "\n";
 
-    $self->reggrp_comments()->exec( $javascript );
-    $self->reggrp_clean()->exec( $javascript );
-    $self->reggrp_whitespace()->exec( $javascript );
-    $self->reggrp_concat()->exec( $javascript );
+    $self->{comments}->{reggrp}->exec( $javascript );
+    $self->{clean}->{reggrp}->exec( $javascript );
+    $self->{whitespace}->{reggrp}->exec( $javascript );
+    $self->{concat}->{reggrp}->exec( $javascript );
 
-    if ( $self->compress() ne 'clean' ) {
-        $self->reggrp_data_store()->exec( $javascript );
+    if ( $opts->{compress} ne 'clean' ) {
+        $self->{data_store}->{reggrp}->exec( $javascript );
 
         while( ${$javascript} =~ /$SHRINK_VARS->{BLOCK}/ ) {
             ${$javascript} =~ s/$SHRINK_VARS->{BLOCK}/$self->_store_block_data( $1 )/egsm;
@@ -408,21 +422,26 @@ sub minify {
             ${$javascript} =~ s/$shrunk_var/$short_id/g;
         }
 
-        $self->reggrp_data_store()->restore_stored( $javascript );
+        $self->{data_store}->{reggrp}->restore_stored( $javascript );
 
         $self->{block_data} = [];
     }
 
-    if ( $self->compress() eq 'obfuscate' or $self->compress() eq 'best' ) {
+    if ( $opts->{compress} eq 'obfuscate' or $opts->{compress} eq 'best' ) {
         my $words = {};
 
         my @words = ${$javascript} =~ /$BASE62_VARS->{WORDS}/g;
 
         my $idx = 0;
 
-        foreach ( @words ) {
-            $words->{$_}->{count}++;
-        }
+        map {
+            if ( exists( $words->{$_} ) ) {
+                $words->{$_}->{count}++;
+            }
+            else {
+                $words->{$_}->{count} = 1;
+            }
+        } @words;
 
         WORD: foreach my $word ( sort { $words->{$b}->{count} <=> $words->{$a}->{count} } keys( %{$words} ) ) {
 
@@ -482,7 +501,7 @@ sub minify {
 
         my $pd = join( '|', @pattern );
 
-        $self->reggrp_trim()->exec( \$pd );
+        $self->{trim}->{reggrp}->exec( \$pd );
 
         unless ( $pd ) {
             $pd = '^$';
@@ -563,14 +582,13 @@ sub minify {
         $packed_length += length( $BASE62_VARS->{UNPACK} );
         $packed_length -= ( $BASE62_VARS->{UNPACK} =~ s/(%s|%d)/$1/g ) * 2;
 
-        my ( @length_matches ) = ${$javascript} =~ s/((?>[\r\n]+))/$1/g;
-        foreach ( @length_matches ) {
+        map {
             $packed_length -= length( $_ ) - 3;
-        }
+        } ${$javascript} =~ s/((?>[\r\n]+))/$1/g;
 
         $packed_length += ${$javascript} =~ tr/\\\'/\\\'/;
 
-        if ( $self->compress() eq 'obfuscate' or $packed_length <= length( ${$javascript} ) ) {
+        if ( $opts->{compress} eq 'obfuscate' or $packed_length <= length( ${$javascript} ) ) {
 
             ${$javascript} =~ s/$BASE62_VARS->{WORDS}/sprintf( "%s", $words->{$1}->{encoded} )/eg;
 
@@ -584,13 +602,17 @@ sub minify {
 
     }
 
-    if ( not $self->remove_copyright() ) {
-        ${$javascript} = ( $self->copyright() || $self->_copyright_comment() ) . ${$javascript};
-    }
+    ${$javascript} = $opts->{copyright} . "\n" . ${$javascript} if ( $opts->{copyright} );
 
     return ${$javascript} if ( $cont eq 'scalar' );
 }
 
+sub _process_wrapper {
+    my ( $self, $reg_name, $in ) = @_;
+
+    $self->{$reg_name}->{reggrp}->exec( $in );
+}
+
 sub _restore_data {
     my ( $self, $string_ref, $data_name, $pattern ) = @_;
 
@@ -706,7 +728,7 @@ JavaScript::Packer - Perl version of Dean Edwards' Packer.js
 
 =head1 VERSION
 
-Version 1.005_001
+Version 1.001
 
 =head1 DESCRIPTION
 
@@ -747,9 +769,11 @@ Default value is 'clean'.
 is recommended because especially when compressing short scripts the result
 will exceed the input if compression level is 'obfuscate'.
 
+For backward compatibility 'minify' and 'base62' will still work.
+
 =item copyright
 
-You can add a copyright notice at the top of the script.
+You can add a copyright notice on top of the script.
 
 =item remove_copyright
 
@@ -767,6 +791,8 @@ prevents the input being packed or defines a compression level.
     /* JavaScript::Packer _no_compress_ */
     /* JavaScript::Packer shrink */
 
+Is set by default.
+
 =back
 
 =head1 EXAMPLES
@@ -829,8 +855,9 @@ Merten Falk, C<< <nevesenin at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests through the web interface at
-L<http://github.com/nevesenin/javascript-packer-perl/issues>.
+Please report any bugs or feature requests to
+C<bug-javascript-packer at rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=JavaScript-Packer>.
 
 =head1 SUPPORT
 
